﻿namespace MurdocTool
{
    using System;
    using System.Collections.Generic;

    enum ErrorLevel
    {
        Success = 0 ,
        MurdocError = 10 ,
        InputError = 20 ,
        WorkError = 30 ,
        Unknown = 255
    }

    class MurdocBatch
    {
        static void Main(string[] args)
        {
            System.Environment.ExitCode = (int)ErrorLevel.Success;

            try
            {
                Options option = Options.Analyze(args);
                var filestatus = option.MurdocFiles.CheckFiles();
                if (filestatus != MurdocFiles.FileStatus.Success)
                {
                    Console.WriteLine(MurdocFiles.GetStatusString(filestatus));
                    System.Environment.ExitCode = 2;
                    return;
                }

                Batches batches = Batches.ReadBatchFile(option.BatchPath);

                if (!option.IncludeClipBin)
                {
                    for (int i = batches.List.Count - 1; i >= 0; i--)
                    {
                        if (batches.List[i].ClipListPath == option.MurdocFiles.ClipBinPath)
                        {
                            batches.List.RemoveAt(i);
                        }
                    }
                }

                var liststatus = batches.CheckList();
                if (liststatus.Status != Batches.CheckResult.FileCheckStatus.Success)
                {
                    Console.WriteLine(liststatus.GetStatusString());
                    foreach (string s in liststatus.ErrorFiles)
                    {
                        Console.WriteLine(s);
                    }
                    System.Environment.ExitCode = (int)ErrorLevel.InputError;
                    return;
                }

                if (batches.List.Count > 0)
                {
                    if (option.StartTime > DateTime.Now)
                    {
                        Console.WriteLine(option.StartTime.ToString() + "まで待機します");
                        while (option.StartTime > DateTime.Now)
                        {
                            Console.SetCursorPosition(0, Console.CursorTop );
                            Console.Write( DateTime.Now.ToString());
                            System.Threading.Thread.Sleep(1000);
                        }
                        Console.WriteLine();
                    }

                    foreach (Batch batch in batches.List)
                    {
                        Console.WriteLine(batch.ClipListPath);
                        if (!option.IsTest)
                        {
                            if (!batch.Action(option))
                            {
                                break;
                            }
                        }
                    }
                }
                if (System.Environment.ExitCode == (int)ErrorLevel.Success)
                {
                    Console.WriteLine("正常に終了");
                }
                if (option.PowerAtFinish != Options.Power.None )
                {
                    API.SetSuspendState(option.PowerAtFinish== Options.Power.Sleep  , false , false);
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                System.Environment.ExitCode = (int)ErrorLevel.Unknown;
            }

        }

    }

    public class Options
    {
        public enum Power
        {
            None = 0 ,Sleep , Standby
        }
        public enum ErrorActionEnum
        {
            None = 0,
            Continue= 1,
            Exit = 2,
        }

        public MurdocFiles MurdocFiles = new MurdocFiles();

        public static string DefaultClippingButtonText = "Clpping";

        public string ClippingButtonText = DefaultClippingButtonText;

        public string BatchPath = System.Environment.CurrentDirectory;
        public bool IsTest = false;
        public bool IncludeClipBin = true;
        public DateTime StartTime = DateTime.MinValue;
        public Power PowerAtFinish = Power.None;
        public ErrorActionEnum ErrorAction = ErrorActionEnum.None;

        public static Options Analyze(string[] args)
        {
            Options aa=new Options();
            int i = 0;
            while (i < args.Length)
            {
                if (args[i][0] == '/')
                {
                    switch (char.ToUpper( args[i][1]))
                    {
                    case 'M':
                        i++;
                        aa.MurdocFiles.SetPath(args[i]);
                        break;

                    case 'B':
                        i++;
                        aa.BatchPath = args[i];
                        break;

                    case 'T':
                        aa.IsTest = true;
                        break;

                    case 'D':
                        i++;
                        aa.StartTime = DateTime.Parse(args[i]);
                        break;

                    case 'S':
                        if (args[i].StartsWith("/SL" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.PowerAtFinish = Power.Sleep;
                        }
                        else if (args[i].StartsWith("/ST" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.PowerAtFinish = Power.Standby;
                        }
                         break;

                    case 'C':
                        i++;
                        aa.ClippingButtonText = args[i];
                        break;

                    case 'E':
                        if (args[i].StartsWith("/EC" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.ErrorAction = ErrorActionEnum.Continue;//続行
                        }
                        else if (args[i].StartsWith("/EE" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.ErrorAction = ErrorActionEnum.Exit;//終了
                        }
                        else if (args[i].StartsWith("/EN" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.ErrorAction = ErrorActionEnum.None; //なにもしない
                        }
                        break;

                    case 'F':
                        if (args[i].StartsWith("/FI" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.IncludeClipBin = true;//Clip.binも処理する
                        }
                        else if (args[i].StartsWith("/FE" , StringComparison.OrdinalIgnoreCase))
                        {
                            aa.IncludeClipBin = false;//Clip.binを処理しない
                        }
                        break;
                    }
                }
                i++;
            }
            return aa;
        }
    }

    public class Batch
    {
        private enum WorkDialogResult
        {
            DialogClose ,
            NotFound ,
            MurdocError ,
            WorkError ,
        }

        public string ClipListPath{get;set;}
        public string OutputPath{get;set;}

        public bool Action(Options option)
        {
            return Action(this , option);
        }

        private static bool Action(Batch batch , Options option)
        {
            Console.WriteLine(" -> " + batch.OutputPath);

            bool retval = true;
            option.MurdocFiles.RewriteOutput_And_CopyClipList(batch);

            System.Diagnostics.ProcessStartInfo pinfo
                = new System.Diagnostics.ProcessStartInfo(option.MurdocFiles.ExePath);

            System.Diagnostics.Process process
                = new System.Diagnostics.Process();
            process.StartInfo = pinfo;
            process.Start();
            process.WaitForInputIdle();

            IntPtr hMurdoc = process.MainWindowHandle;
            IntPtr hClippingButton = IntPtr.Zero;
            hClippingButton = API.FindWindowEx(hMurdoc , hClippingButton , "Button" , option.ClippingButtonText);

            if (hClippingButton == IntPtr.Zero)
            {
                Console.WriteLine("Clippingボタンが見つかりませんでした");
                System.Environment.ExitCode = (int)ErrorLevel.WorkError;
                retval = false;
            }
            else
            {
                WorkDialogResult result = ClickClippingButton(hMurdoc , hClippingButton , option);
                switch(result)
                {
                case WorkDialogResult.MurdocError: //through
                case WorkDialogResult.WorkError:
                    retval = CheckContinue(result,option.ErrorAction);
                    break;

                case WorkDialogResult.NotFound:
                    do
                    {
                        System.Threading.Thread.Sleep(1000);
                        result = WorkDialog(hMurdoc);
                        if(result ==  WorkDialogResult.MurdocError || result == WorkDialogResult.WorkError )
                        {
                            retval = CheckContinue(result , option.ErrorAction);
                            break;
                        }
                    } while (API.GetText(hClippingButton) != option.ClippingButtonText || !API.IsWindowEnabled(hClippingButton));
                    break;
                case WorkDialogResult.DialogClose:
                    break;
                }
            }

            System.Threading.Thread.Sleep(1000);
            //API.SendMessage(hMurdoc , (uint)API.WM.WM_CLOSE , IntPtr.Zero , IntPtr.Zero);
            process.CloseMainWindow();//
            if (!process.WaitForExit(10000))
            {
                Console.WriteLine("閉じることが出来ませんでした");
                System.Environment.ExitCode = (int)ErrorLevel.WorkError;
                retval = false;
            }

            return retval;
        }
        private static bool CheckContinue(WorkDialogResult workresult , Options.ErrorActionEnum errorAction)
        {
            switch (workresult)
            {
            case WorkDialogResult.DialogClose:
                return true;
            case WorkDialogResult.WorkError:
                Console.WriteLine("MurdocCutterが想定外の動作をしています");
                System.Environment.ExitCode = (int)ErrorLevel.WorkError;
                return false;
            case WorkDialogResult.MurdocError:
                Console.WriteLine("MurdocCutterでエラーが表示されました");
                if (errorAction != Options.ErrorActionEnum.None)
                {
                    System.Environment.ExitCode = (int)ErrorLevel.MurdocError;
                }
                if (errorAction != Options.ErrorActionEnum.Exit)
                {
                    return true;
                }
                else
                {
                    Console.WriteLine("処理を中止します。");
                    return false;
                }
            case WorkDialogResult.NotFound:
            default:
                return true;
            }
        }


        /// <summary></summary>
        /// <param name="hMurdoc"></param>
        /// <param name="hClippingButton"></param>
        /// <param name="analyze"></param>
        /// <returns>エラーダイアログが表示されて、続行しない場合はfalse</returns>
        private static WorkDialogResult ClickClippingButton(IntPtr hMurdoc , IntPtr hClippingButton , Options option)
        {
            bool dialogcheck = false;
            System.Threading.ParameterizedThreadStart start
                = new System.Threading.ParameterizedThreadStart(
                delegate(object param)
                {//エラーダイアログを閉じるためのスレッド
                    object[] values = (object[])param;
                    IntPtr _hMurdoc = (IntPtr)values[0];
                    Options _analyze = (Options)values[1];

                    while (!dialogcheck)
                    {
                        System.Threading.Thread.Sleep(100);
                        WorkDialogResult wr;
                        wr = WorkDialog(_hMurdoc);
                        if (wr != WorkDialogResult.NotFound)
                        {
                            values[2] = wr;
                            break;
                        }
                    }
                });

            object[] threadParam = new object[] { hMurdoc , option , WorkDialogResult.NotFound };
            System.Threading.Thread thread = new System.Threading.Thread(start);
            thread.IsBackground = true;
            thread.Start(threadParam);

            API.SendMessage(hClippingButton , (uint)API.WM.WM_LBUTTONDOWN , IntPtr.Zero , IntPtr.Zero);
            API.SendMessage(hClippingButton , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);//SendMessageだとエラーのときに止まる
            dialogcheck = true;
            thread.Join();
            return (WorkDialogResult)threadParam[2];
        }

        private class FindDialog
        {
            public static Dictionary<IntPtr , string> FindDialogs(IntPtr hOwner)
            {
                FindDialog fd = new FindDialog();
                fd.hOwner = hOwner;
                API.GetWindowThreadProcessId(hOwner , out fd.processID);
                API.EnumWindows(fd.EnumDialogProc , IntPtr.Zero);

                return fd.dic;
            }

            private IntPtr hOwner;
            private uint processID;
            private Dictionary<IntPtr , string> dic = new Dictionary<IntPtr , string>();

            private bool EnumDialogProc(IntPtr hWnd , IntPtr lParam)
            {
                if (hWnd != hOwner)
                {//メインウィンドウではなくInputウィンドウが親になってる場合がある
                    uint pid;
                    API.GetWindowThreadProcessId(hWnd , out pid);
                    if (pid == processID && API.GetClassName(hWnd) == "#32770")
                    {
                        string title = API.GetText(hWnd);
                        if (title != "View" && title != "Input")
                        {
                            dic.Add(hWnd , title);
                        }
                    }
                }
                return true;
            }
        }


        /// <summary>ダイアログの情報を取って閉じる</summary>
        /// <param name="hMurdoc"></param>
        /// <param name="hDialog"></param>
        /// <param name="erraction"></param>
        /// <returns>エラーダイアログが表示されて、続行しない場合はfalse</returns>
        private static WorkDialogResult WorkDialog(IntPtr hMurdoc)//,ArgAnalyze.ErrorActionEnum erraction)
        {
            Dictionary<IntPtr , string> dialogs = FindDialog.FindDialogs(hMurdoc);

            if (dialogs.Count == 0)
            {
                return WorkDialogResult.NotFound;
            }
            else if (dialogs.Count > 1)
            {
                //ダイアログがいっぱい？
                return WorkDialogResult.WorkError;
            }

            foreach (IntPtr hDialog in dialogs.Keys)
            {
                IntPtr hEdit;
                IntPtr hOKButton;

                hEdit = API.FindWindowEx(hDialog , IntPtr.Zero , "Edit" , null);
                hOKButton = API.FindWindowEx(hDialog , IntPtr.Zero , "Button" , "OK");
                if (hOKButton != IntPtr.Zero)
                {
                    string strTitle = dialogs[hDialog];
                    Console.WriteLine(dialogs[hDialog]);
                    if (hEdit != IntPtr.Zero)
                    {
                        Console.WriteLine(API.GetText(hEdit));
                    }

                    IntPtr sendResult;
                    int trycount = 0;
                    while (API.IsWindow(hDialog))
                    {
                        ////オプションで output elapsed time display が有効になってると
                        ////経過時間ウィンドウが表示されてるので閉じないといけない
                        sendResult = API.SendMessage(hOKButton , (uint)API.WM.WM_LBUTTONDOWN , IntPtr.Zero , IntPtr.Zero);
                        sendResult = API.SendMessage(hOKButton , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);
                        System.Threading.Thread.Sleep(100);
                        if (++trycount > 100)
                        {
                            return WorkDialogResult.WorkError;
                        }
                    }

                    if (strTitle != "Info")
                    {//エラーウィンドウが表示されることもあるよ
                        return WorkDialogResult.MurdocError;
                    }
                    return WorkDialogResult.DialogClose;
                }
            }

            return WorkDialogResult.WorkError;


        }
 
    }

    public class Batches
    {
        private Batches()
        {
        }

        public List<Batch> List
        {
            get { return _List; }
        }
        private List<Batch> _List = new List<Batch>();

        public static Batches ReadBatchFile(string batchPath)
        {
            Batches bats = new Batches();

            batchPath = batchPath.Trim();

            if (System.IO.Directory.Exists(batchPath))
            {
                foreach (string path in System.IO.Directory.GetFiles(batchPath , "*.bin" , System.IO.SearchOption.TopDirectoryOnly))
                {
                    if (System.IO.Path.GetExtension(path).Equals(".bin" , StringComparison.OrdinalIgnoreCase))
                    {
                        Batch bat = new Batch();
                        bat.ClipListPath = path;
                        bat.OutputPath = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(path)
                                                              , System.IO.Path.GetFileNameWithoutExtension(path) + ".ts");
                        bats.List.Add(bat);
                    }
                }
            }
            else if (System.IO.File.Exists(batchPath))
            {
                using (System.IO.StreamReader sr = new System.IO.StreamReader(batchPath , System.Text.Encoding.Default))
                {
                    while (sr.Peek() != -1)
                    {
                        string line;
                        line = sr.ReadLine();
                        string[] datas = line.Split('|');
                        for (int i = 0; i < datas.Length; i++)
                        {
                            datas[i] = datas[i].Trim();
                            if (datas[i].StartsWith("\"") && datas[i].EndsWith("\""))
                            {
                                datas[i] = datas[i].Substring(1 , datas[i].Length - 2);
                            }
                        }
                        if (datas.Length >= 2)
                        {
                            Batch bat = new Batch();
                            bat.ClipListPath = datas[0];
                            bat.OutputPath = datas[1];


                            bats.List.Add(bat);
                        }
                    }
                }
            }
            return bats;
        }

        public List<string> ClipListNotExists
        {
            get
            {
                List<string> retval = new List<string>();
                foreach (Batch batch in this.List)
                {
                    if (!System.IO.File.Exists(batch.ClipListPath))
                    {
                        retval.Add(batch.ClipListPath);
                    }
                }
                return retval;
            }
        }

        public List<string> OutputExists
        {
            get
            {
                List<string> retval = new List<string>();
                foreach (Batch batch in this.List)
                {
                    if (System.IO.File.Exists(batch.OutputPath))
                    {
                        retval.Add(batch.OutputPath);
                    }
                }
                return retval;
            }
        }

        public List<string> HasDoubleOutputPath
        {
            get
            {
                System.Collections.Generic.List<string> list = new List<string>();
                System.Collections.Generic.List<string> retval = new List<string>();
                foreach (Batch batch in List)
                {
                    if (list.Contains(batch.OutputPath))
                    {
                        retval.Add(batch.OutputPath);
                    }
                    list.Add(batch.OutputPath);
                }

                return retval;
            }
        }
        public void DeleteExists()
        {
            foreach (Batch batch in this.List)
            {
                if (System.IO.File.Exists(batch.OutputPath))
                {
                    System.IO.File.Delete(batch.OutputPath);
                }
            }
        }


        public class CheckResult
        {
             public enum FileCheckStatus
            {
                Success = 0 ,
                NotList ,
                Exists ,
                Double ,
            }
            public static string GetStatusString(FileCheckStatus status)
            {
                switch (status)
                {
                case FileCheckStatus.Success:
                    return "OK";
                case FileCheckStatus.NotList:
                    return "ClipListが見つかりません";
                case FileCheckStatus.Exists:
                    return "出力先が既に存在しているためキャンセルしました";
                case FileCheckStatus.Double:
                    return "出力先が重複しているためキャンセルしました";
                }
                return string.Empty;
            }

            public string GetStatusString()
            {
                return GetStatusString(this.Status);
            }

            public FileCheckStatus Status = FileCheckStatus.Success;
            public List<string> ErrorFiles = new List<string>();
        }

        public CheckResult CheckList()
        {
            CheckResult retval = new CheckResult();
            retval.ErrorFiles  = this.ClipListNotExists;
            if (retval.ErrorFiles.Count > 0)
            {
                retval.Status = CheckResult.FileCheckStatus.NotList;
                return retval ;
            }

            retval.ErrorFiles = this.OutputExists;
            if (retval.ErrorFiles.Count > 0)
            {
                retval.Status = CheckResult.FileCheckStatus.Exists;
                return retval;
            }

            retval.ErrorFiles = this.HasDoubleOutputPath;
            if (retval.ErrorFiles.Count > 0)
            {
                retval.Status = CheckResult.FileCheckStatus.Double;
                return retval;
            }
            return retval;
        }
    }

    public class MurdocFiles
    {
        public const string DEFAULT_EXE_FILENAME = "MurdocCut.exe";

        public MurdocFiles()
        {
            SetPath(DEFAULT_EXE_FILENAME);
        }

        public void SetPath(string exePath)
        {
            System.IO.FileInfo fi = new System.IO.FileInfo(exePath);
            this.ExePath = fi.FullName;
            this.IniPath = System.IO.Path.Combine(fi.Directory.FullName  , "MurdocCut.ini");
            this.ClipBinPath = System.IO.Path.Combine(fi.Directory.FullName , "Clip.bin");
        }

        public string ExePath { get; private set; }
        public string IniPath { get; private set; }
        public string ClipBinPath { get; private set; }

        public enum FileStatus
        {
            Success= 0,
            NotFound ,
            NotEXE ,
            NotMurdocCutter,
            NotSupportedVersion,
        }

        public static string GetStatusString(FileStatus status)
        {
            switch (status)
            {
            case FileStatus.Success:
                return "OK";
            case FileStatus.NotFound:
                return "MurdocCutterが見つかりません";
            case FileStatus.NotEXE:
                return "MurdocCutterのパスがexeではありません";
            case FileStatus.NotMurdocCutter:
                return "MurdocCutterと認識できませんでした";
            case FileStatus.NotSupportedVersion:
                return "MurdocCutterのバージョンが古いです";
            }
            return string.Empty;
        }

        public FileStatus CheckFiles()
        {
            FileStatus retval = FileStatus.Success;
            if (!System.IO.File.Exists(ExePath))
            {
                retval = FileStatus.NotFound;
            }
            else if (!System.IO.Path.GetExtension(ExePath).Equals(".exe" , System.StringComparison.OrdinalIgnoreCase))
            {
                retval = FileStatus.NotEXE;
            }
            else
            {
                System.Diagnostics.FileVersionInfo fvi = System.Diagnostics.FileVersionInfo.GetVersionInfo(ExePath);
                System.Version ver = new System.Version(fvi.FileMajorPart , fvi.FileMinorPart , fvi.FileBuildPart , fvi.FilePrivatePart);
                if (fvi.ProductName != "Murdoc Cutter")
                {
                    retval = FileStatus.NotMurdocCutter;
                }
                else if (ver < new Version(1 , 9 , 8 , 1))
                {
                    retval = FileStatus.NotSupportedVersion;
                }
            }
            return retval;
        }

        public bool IsEnableShowElapsedWindow
        {
            get
            {

                return _IsEnableShowElapsedWindow;
            }
        }
        private bool _IsEnableShowElapsedWindow = false;

        public void RewriteOutputPath(string outputPath)
        {
            if (!API.WritePrivateProfileString("FILE" , "OUTPUT" , outputPath , this.IniPath))
            {
                throw new System.IO.IOException("INIファイルに書き込み出来ませんでした");
            }
        }

        public void CopyClipList(string newClipListPath)
        {
            System.IO.FileInfo f1 = new System.IO.FileInfo(newClipListPath);
            System.IO.FileInfo f2 = new System.IO.FileInfo(this.ClipBinPath);
            if (!f1.FullName.Equals(f2.FullName, StringComparison.OrdinalIgnoreCase)) 
            {
                System.IO.File.Copy(newClipListPath , this.ClipBinPath , true);
            }
        }

        public void RewriteOutput_And_CopyClipList(Batch batch)
        {
            RewriteOutputPath(batch.OutputPath);
            CopyClipList(batch.ClipListPath );
            _IsEnableShowElapsedWindow = ("0" != API.GetPrivateProfileString("FILE" , "OUTTIME" , "0" , this.IniPath));
        }
    }
}

namespace MurdocTool
{
    using System;

    public class MurdocWindow : System.Windows.Forms.IWin32Window
    {
        public MurdocWindow(System.Diagnostics.Process process) : this(process,Options.DefaultClippingButtonText)
        {
        }

        public MurdocWindow(System.Diagnostics.Process process , string clippingButtonText)
        {


            this._process = process;
            this.hWnd = process.MainWindowHandle; ;

            hSaveButton = API.FindWindowEx(hWnd , IntPtr.Zero , "Button" , "Save");
            hClippingButton = API.FindWindowEx(hWnd , IntPtr.Zero , "Button" , clippingButtonText);//"Clpping");

            hDeleteButton = API.FindWindowEx(hWnd , IntPtr.Zero , "Button" , "×");
            hClearButton = API.FindWindowEx(hWnd , IntPtr.Zero , "Button" , "Clear");
            hListBox = API.FindWindowEx(hWnd , IntPtr.Zero , "ListBox" , null);

            IntPtr h = IntPtr.Zero;
            for (; ; )
            {
                h = API.FindWindowEx(hWnd , h , "Edit" , null);
                if (h == IntPtr.Zero)
                {
                    break;
                }
                uint style = (uint)(uint)API.GetWindowLong_Style(h);
                if ((style & (uint)API.WINDOW_STYLE.WS_VISIBLE) == (uint)API.WINDOW_STYLE.WS_VISIBLE)
                {
                    if ((style & (uint)API.EDITBOX_STYLE.ES_READONLY)
                        == (uint)API.EDITBOX_STYLE.ES_READONLY)
                    {

                        hOriginalTextBox = h;
                    }
                    else
                    {
                        hOutputTextBox = h;
                    }
                }
            }
        }

        private System.Diagnostics.Process _process;
        private IntPtr hWnd;
        private IntPtr hSaveButton;
        private IntPtr hClippingButton;
        private IntPtr hOriginalTextBox;
        private IntPtr hOutputTextBox;
        private IntPtr hClearButton;
        private IntPtr hDeleteButton;
        private IntPtr hListBox;

        public System.Diagnostics.Process Process
        {
            get { return this._process; }
        }

        public IntPtr Handle
        {
            get { return hWnd; }
        }

        private void Click(IntPtr hwnd)
        {
            API.SendMessage(hwnd , (uint)API.WM.WM_LBUTTONDOWN , IntPtr.Zero , IntPtr.Zero);
            API.SendMessage(hwnd , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);

        }
        public void ClickSaveButton()
        {
            //API.SendMessage(hSaveButton , 0x21 , hWnd , new IntPtr(0x02010001));
            API.SendMessage(hSaveButton , (uint)API.WM.WM_LBUTTONDOWN , new IntPtr(1) , IntPtr.Zero);
            API.PostMessage(hSaveButton , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);//SendMessageだとエラーのときに止まる

        }
        public void ClickClipButton()
        {
            Click(hClippingButton);
        }

        public void ClickClearButton()
        {
            Click(hClearButton);
        }

        public void ClickDeleteButton()
        {
            Click(hDeleteButton);
        }

        public bool ClippingButtonEnabled
        {
            get
            {
                return (API.GetWindowLong_Style(hClippingButton) & API.WINDOW_STYLE.WS_DISABLED)
                    != API.WINDOW_STYLE.WS_DISABLED;
            }
        }

        public string ClippingButtonText
        {
            get { return API.GetText(hClippingButton); }
        }

        public string OriginalPath
        {
            get
            {
                return API.GetText(hOriginalTextBox);
            }
        }
        public string OutputPath
        {
            get
            {
                return API.GetText(hOutputTextBox);
            }
            set
            {
                API.SetText(hOutputTextBox , value);
            }
        }

        public int ListCount
        {
            get
            {
                uint LB_GETCOUNT = 0x18b;
                return API.SendMessage(hListBox , LB_GETCOUNT , IntPtr.Zero , IntPtr.Zero).ToInt32();
            }
        }

        public System.Collections.Generic.List<string> GetList()
        {
            System.Collections.Generic.List<string> retval
                = new System.Collections.Generic.List<string>();
            int count = ListCount;
            uint LB_GETTEXT = 0x189;
            uint LB_GETTEXTLEN = 0x18a;
            for (int i = 0; i < count; i++)
            {

                System.Text.StringBuilder sb;
                int size = (int)API.SendMessage(hListBox , LB_GETTEXTLEN , i , null);
                sb = new System.Text.StringBuilder(size);
                API.SendMessage(hListBox , LB_GETTEXT , i , sb);
                retval.Add(sb.ToString());

            }
            return retval;
        }

        public System.Collections.Generic.List<ListData> GetListData()
        {
            return ListData.GetListData(GetList());
        }

        public bool HasForcusNewPath
        {
            get
            {
                IntPtr hwndForcus = IntPtr.Zero;
                uint pid;
                uint targetThreadID = API.GetWindowThreadProcessId(hWnd , out pid);
                uint thisThreadID = API.GetCurrentThreadId();

                if (API.AttachThreadInput(thisThreadID , targetThreadID , true))
                {
                    hwndForcus = API.GetFocus();
                    API.AttachThreadInput(thisThreadID , targetThreadID , false);
                }

                return hwndForcus == hOutputTextBox;
            }
        }
    }

    public class ListData
    {
        public TimeSpan  Start;
        public TimeSpan  End;
        public string File;

        public static System.Collections.Generic.List<ListData> GetListData(System.Collections.Generic.IEnumerable<string> list)
        {
            System.Collections.Generic.List<ListData> retval 
                = new System.Collections.Generic.List<ListData>();
            foreach (string s in list)
            {
                ListData ld = new ListData();
                ld.Start = GetSpan(s.Substring(0 , 12));
                ld.End = GetSpan(s.Substring(15 , 12));
                ld.File = s.Substring(28).Trim();
                retval.Add(ld);
            }
            return retval;
        }

        private static TimeSpan GetSpan(string s)
        {

            return TimeSpan.Parse(s.Substring(0 , 7) + "." + s.Substring(9));
            //return DateTime.ParseExact(s , "HH:mm:ss:fff"
            //    , System.Threading.Thread.CurrentThread.CurrentCulture , System.Globalization.DateTimeStyles.NoCurrentDateDefault);
        }
    }
}

namespace MurdocTool
{
    using System;
    using System.Runtime.InteropServices;

    public class API
    {
        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr FindWindowEx(IntPtr hwndParent , IntPtr hwndChildAfter , string lpszClass , string lpszWindow);

        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr FindWindowEx(IntPtr parentHandle , IntPtr childAfter , string className , IntPtr windowTitle);

        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(IntPtr hWnd , UInt32 Msg , IntPtr wParam , IntPtr lParam);

        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(IntPtr hWnd , UInt32 Msg , int wParam , System.Text.StringBuilder lParam);

        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr PostMessage(IntPtr hWnd , UInt32 Msg , IntPtr wParam , IntPtr lParam);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool IsWindowEnabled(IntPtr hWnd);

        [DllImport("Powrprof.dll" , SetLastError = true)]
        public static extern bool SetSuspendState(bool hibernate , bool forceCritical , bool disableWakeEvent);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lpSectionName"></param>
        /// <param name="lpKeyName"></param>
        /// <param name="lpString"></param>
        /// <param name="lpFileName">フルパスで</param>
        /// <returns></returns>
        [DllImport("kernel32.dll" , CharSet = CharSet.Auto  , SetLastError = true)]
        public static extern bool WritePrivateProfileString(string lpSectionName , string lpKeyName , string lpString , string lpFileName);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lpSectionName"></param>
        /// <param name="lpKeyName"></param>
        /// <param name="lpDefault"></param>
        /// <param name="lpReturnedString"></param>
        /// <param name="nSize"></param>
        /// <param name="lpFileName">フルパスで</param>
        /// <returns></returns>
        [DllImport("kernel32.dll" , CharSet = CharSet.Auto)]
        static extern uint GetPrivateProfileString(string lpSectionName , string lpKeyName , string lpDefault , System.Text.StringBuilder lpReturnedString , uint nSize , string lpFileName);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lpSectionName"></param>
        /// <param name="lpKeyName"></param>
        /// <param name="lpDefault"></param>
        /// <param name="lpFileName">フルパスで</param>
        /// <returns></returns>
        public static string GetPrivateProfileString(string lpSectionName , string lpKeyName , string lpDefault , string lpFileName)
        {
            System.Text.StringBuilder sb=new System.Text.StringBuilder(64 * 1024);
            uint length =GetPrivateProfileString(lpSectionName , lpKeyName , lpDefault , sb , (uint)sb.Capacity  , lpFileName);
            return sb.ToString();
        }

        [DllImport("user32.dll" , SetLastError = true , CharSet = CharSet.Auto)]
        public static extern int GetClassName(IntPtr hWnd , System.Text.StringBuilder lpClassName , int nMaxCount);

        public static string GetClassName(IntPtr hWnd)
        {
            System.Text.StringBuilder sb = new System.Text.StringBuilder(64);
            int length = GetClassName(hWnd , sb , sb.Capacity);
            return sb.ToString();
        }

        public static string GetText(IntPtr hwnd)
        {
            int size;
            size = SendMessage(hwnd , (uint)WM.WM_GETTEXTLENGTH , IntPtr.Zero , IntPtr.Zero).ToInt32();
            System.Text.StringBuilder sb = new System.Text.StringBuilder(size + 1);
            SendMessage(hwnd , (uint)WM.WM_GETTEXT , size + 1 , sb);
            return sb.ToString();
        }
        public static void SetText(IntPtr hwnd , string text)
        {
            System.Text.StringBuilder sb = new System.Text.StringBuilder(text);
            API.SendMessage(hwnd , (uint)API.WM.WM_SETTEXT , 0 , sb);
        }
        public enum WM : uint
        {
            WM_GETTEXTLENGTH = 0x000E ,
            WM_SETTEXT = 0x000C,
            WM_GETTEXT = 0x000D ,
            WM_CLOSE = 0x10 ,
            WM_LBUTTONDOWN = 0x0201 ,
            WM_LBUTTONUP = 0x0202
        }

        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr GetWindow(IntPtr hWnd , GetWindow_Cmd uCmd);

        public enum GetWindow_Cmd : uint
        {
            GW_HWNDFIRST = 0 ,
            GW_HWNDLAST = 1 ,
            GW_HWNDNEXT = 2 ,
            GW_HWNDPREV = 3 ,
            GW_OWNER = 4 ,
            GW_CHILD = 5 ,
            GW_ENABLEDPOPUP = 6
        }

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool IsWindowVisible(IntPtr hWnd);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool IsWindow(IntPtr hWnd);

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool EnumWindows(EnumWindowsProc lpEnumFunc , IntPtr lParam);

        public delegate bool EnumWindowsProc(IntPtr hWnd , IntPtr lParam);

        [DllImport("user32.dll" , SetLastError = true)]
        public static extern uint GetWindowThreadProcessId(IntPtr hWnd , out uint lpdwProcessId);


        [DllImport("user32.dll" , SetLastError = true)]
        public static extern uint GetWindowLong(IntPtr hWnd , GETWINDOWLONG_INDEX nIndex);

        public static WINDOW_STYLE GetWindowLong_Style(IntPtr hWnd)
        {
            return (WINDOW_STYLE)GetWindowLong(hWnd , GETWINDOWLONG_INDEX.GWL_STYLE);
        }
        public static WINDOW_EXSTYLE GetWindowLong_ExStyle(IntPtr hWnd)
        {
            return (WINDOW_EXSTYLE)GetWindowLong(hWnd , GETWINDOWLONG_INDEX.GWL_EXSTYLE);
        }

        public enum GETWINDOWLONG_INDEX
        {
            GWL_ID = (-12) ,
            GWL_STYLE = (-16) ,
            GWL_EXSTYLE = (-20) ,
        }

        [FlagsAttribute ]
        public enum WINDOW_STYLE : uint
        {
            WS_OVERLAPPED = 0 ,
            WS_POPUP = 0x80000000 ,
            WS_CHILD = 0x40000000 ,
            WS_MINIMIZE = 0x20000000 ,
            WS_VISIBLE = 0x10000000 ,
            WS_DISABLED = 0x8000000 ,
            WS_CLIPSIBLINGS = 0x4000000 ,
            WS_CLIPCHILDREN = 0x2000000 ,
            WS_MAXIMIZE = 0x1000000 ,
            WS_CAPTION = 0xC00000 ,      // WS_BORDER or WS_DLGFRAME  
            WS_BORDER = 0x800000 ,
            WS_DLGFRAME = 0x400000 ,
            WS_VSCROLL = 0x200000 ,
            WS_HSCROLL = 0x100000 ,
            WS_SYSMENU = 0x80000 ,
            WS_THICKFRAME = 0x40000 ,
            WS_GROUP = 0x20000 ,
            WS_TABSTOP = 0x10000 ,
            WS_MINIMIZEBOX = 0x20000 ,
            WS_MAXIMIZEBOX = 0x10000 ,
            WS_TILED = WS_OVERLAPPED ,
            WS_ICONIC = WS_MINIMIZE ,
            WS_SIZEBOX = WS_THICKFRAME ,
        }

        [FlagsAttribute]
        public enum WINDOW_EXSTYLE : uint
        {
            WS_EX_DLGMODALFRAME = 0x0001 ,
            WS_EX_NOPARENTNOTIFY = 0x0004 ,
            WS_EX_TOPMOST = 0x0008 ,
            WS_EX_ACCEPTFILES = 0x0010 ,
            WS_EX_TRANSPARENT = 0x0020 ,
            WS_EX_MDICHILD = 0x0040 ,
            WS_EX_TOOLWINDOW = 0x0080 ,
            WS_EX_WINDOWEDGE = 0x0100 ,
            WS_EX_CLIENTEDGE = 0x0200 ,
            WS_EX_CONTEXTHELP = 0x0400 ,
            WS_EX_RIGHT = 0x1000 ,
            WS_EX_LEFT = 0x0000 ,
            WS_EX_RTLREADING = 0x2000 ,
            WS_EX_LTRREADING = 0x0000 ,
            WS_EX_LEFTSCROLLBAR = 0x4000 ,
            WS_EX_RIGHTSCROLLBAR = 0x0000 ,
            WS_EX_CONTROLPARENT = 0x10000 ,
            WS_EX_STATICEDGE = 0x20000 ,
            WS_EX_APPWINDOW = 0x40000 ,
            WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE) ,
            WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE | WS_EX_TOOLWINDOW | WS_EX_TOPMOST) ,
            WS_EX_LAYERED = 0x00080000 ,
            WS_EX_NOINHERITLAYOUT = 0x00100000 , // Disable inheritence of mirroring by children
            WS_EX_LAYOUTRTL = 0x00400000 , // Right to left mirroring
            WS_EX_COMPOSITED = 0x02000000 ,
            WS_EX_NOACTIVATE = 0x08000000 ,
        }

        [FlagsAttribute]
        public enum EDITBOX_STYLE : uint
        {
            ES_READONLY = 0x800
        }

        [DllImport("user32.dll")]
        public static extern IntPtr GetDlgItem(IntPtr hDlg , int nIDDlgItem);

        [System.Runtime.InteropServices.DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern MessageBoxResult MessageBox(IntPtr hWnd , String text , String caption , MessageBoxOptions options);
        [Flags]
        public enum MessageBoxOptions : uint
        {
            Ok = 0x000000 ,
            OkCancel = 0x000001 ,
            AbortRetryIgnore = 0x000002 ,
            YesNoCancel = 0x000003 ,
            YesNo = 0x000004 ,
            RetryCancel = 0x000005 ,
            CancelTryContinue = 0x000006 ,

            IconHand = 0x000010 ,
            IconQuestion = 0x000020 ,
            IconExclamation = 0x000030 ,
            IconAsterisk = 0x000040 ,
            UserIcon = 0x000080 ,

            IconWarning = IconExclamation ,
            IconError = IconHand ,
            IconInformation = IconAsterisk ,
            IconStop = IconHand ,

            DefButton1 = 0x000000 ,
            DefButton2 = 0x000100 ,
            DefButton3 = 0x000200 ,
            DefButton4 = 0x000300 ,

            ApplicationModal = 0x000000 ,
            SystemModal = 0x001000 ,
            TaskModal = 0x002000 ,

            Help = 0x004000 , //Help Button
            NoFocus = 0x008000 ,

            SetForeground = 0x010000 ,
            DefaultDesktopOnly = 0x020000 ,
            Topmost = 0x040000 ,
            Right = 0x080000 ,
            RTLReading = 0x100000 ,
        }

        /// <summary>
        /// Represents possible values returned by the MessageBox function.
        /// </summary>
        public enum MessageBoxResult : uint
        {
            Ok = 1 ,
            Cancel ,
            Abort ,
            Retry ,
            Ignore ,
            Yes ,
            No ,
            Close ,
            Help ,
            TryAgain ,
            Continue ,
            Timeout = 32000
        }

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool AttachThreadInput(uint idAttach , uint idAttachTo ,
           bool fAttach);
        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern IntPtr GetFocus();
        [System.Runtime.InteropServices.DllImport("kernel32.dll")]
        public static extern uint GetCurrentThreadId();

        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr SetActiveWindow(IntPtr hWnd);
    }
}
